from typing import Generator
from unittest import mock

import pytest
import responses
from responses import matchers

from globus_sdk import AuthClient, RefreshTokenAuthorizer, TransferClient

from gcs_migration.typing.transfer_documents import (
    AccessDocumentDict,
    AccessListDocumentDict,
    AnonymousEffectivePermissionsDict,
    EndpointDocumentDict,
    EndpointServerDocumentDict,
    RoleDocumentDict,
    RoleListDocumentDict,
    SharedEndpointListDict,
)
from gcs_migration.typing.auth_documents import (
    IdentityDocumentDict,
    IdentityProviderDocumentDict,
)


@pytest.fixture(autouse=True)
def mock_exponential_backoff() -> Generator[None, None, None]:
    with mock.patch("globus_sdk.transport.requests.time"):
        yield


@pytest.fixture
def mock_responses() -> Generator[None, None, None]:
    responses.start()
    yield
    responses.stop()
    responses.reset()


@pytest.fixture
def mock_tokens(mock_responses: None) -> Generator[None, None, None]:
    with mock.patch.object(RefreshTokenAuthorizer, "__new__", return_value=None):
        yield


@pytest.fixture
def mock_get_transfer_api(
    mock_tokens: None,
    host_endpoint_uuid: str,
    share_endpoint_uuid: str,
    access_acl_document: AccessDocumentDict,
    endpoint_server_document: EndpointServerDocumentDict,
    host_endpoint_document: EndpointDocumentDict,
    shared_endpoint_document: EndpointDocumentDict,
    role_document: RoleDocumentDict,
    role_list_document: RoleListDocumentDict,
    share_role_list_document: RoleListDocumentDict,
    anonymous_effective_permissions_document: AnonymousEffectivePermissionsDict,
    access_list_document: AccessListDocumentDict,
    shared_endpoint_list_document: SharedEndpointListDict,
) -> None:
    tc = TransferClient()

    responses.add(
        "GET",
        f"{tc.base_url}endpoint/{host_endpoint_uuid}",
        json=host_endpoint_document,
    )
    responses.add(
        "GET",
        f"{tc.base_url}endpoint/{share_endpoint_uuid}",
        json=shared_endpoint_document,
    )
    responses.add(
        "GET",
        f"{tc.base_url}endpoint/{host_endpoint_uuid}/role_list",
        json=role_list_document,
    )
    responses.add(
        "GET",
        f"{tc.base_url}endpoint/{host_endpoint_uuid}/shared_endpoint_list",
        json=shared_endpoint_list_document,
    )
    responses.add(
        "GET",
        f"{tc.base_url}endpoint_manager/endpoint/{share_endpoint_uuid}/access_list",
        json=access_list_document,
    )
    responses.add(
        "GET",
        f"{tc.base_url}endpoint/{share_endpoint_uuid}/role_list",
        json=share_role_list_document,
    )
    responses.add(
        "GET",
        f"{tc.base_url}private/endpoint/{share_endpoint_uuid}/anonymous_effective_permissions",
        json=anonymous_effective_permissions_document,
    )
    return None


@pytest.fixture
def unknown_identity_id() -> str:
    return "3f2c5a46-2b93-413d-a0ce-95bd13d075a0"


@pytest.fixture
def mock_get_auth_api(
    mock_tokens: None,
    globus_identity_provider_document: IdentityProviderDocumentDict,
    email_identity_provider_document: IdentityProviderDocumentDict,
    example_identity_provider_document: IdentityProviderDocumentDict,
    identity_document: IdentityDocumentDict,
    unused_idp_identity_document: IdentityDocumentDict,
    closed_idp_identity_document: IdentityDocumentDict,
    sharing_local_identity_document: IdentityDocumentDict,
    email_idp_identity_document: IdentityDocumentDict,
    unknown_identity_id: str,
) -> None:
    ac = AuthClient()

    # commands/gridmap
    responses.add(
        "GET",
        f"{ac.base_url}v2/api/identities",
        match=[
            matchers.query_param_matcher(
                {"usernames": identity_document["username"], "provision": "false"}
            ),
        ],
        json={"identities": [identity_document]},
    )
    # commands/gridmap
    responses.add(
        "GET",
        f"{ac.base_url}v2/api/identities",
        match=[
            matchers.query_param_matcher(
                {
                    "usernames": email_idp_identity_document["username"],
                    "provision": "false",
                }
            )
        ],
        json={"identities": [email_idp_identity_document]},
    )
    # commands/gridmap
    responses.add(
        "GET",
        f"{ac.base_url}v2/api/identities",
        match=[
            matchers.query_param_matcher(
                {"usernames": "unknownuser@globus.org", "provision": "false"}
            )
        ],
        json={"identities": [identity_document]},
    )

    # check/guest_collection
    responses.add(
        "GET",
        f"{ac.base_url}v2/api/identities",
        match=[matchers.query_param_matcher({"ids": identity_document["id"]})],
        json={"identities": [identity_document]},
    )
    # check/guest_collection
    responses.add(
        "GET",
        f"{ac.base_url}v2/api/identities",
        match=[matchers.query_param_matcher({"ids": "not-a-uuid"})],
        json={"identities": []},
    )
    # check/guest_collection
    responses.add(
        "GET",
        f"{ac.base_url}v2/api/identities",
        match=[matchers.query_param_matcher({"ids": "unknown_identity_id}"})],
        json={"identities": []},
    )
    # check/guest_collection
    responses.add(
        "GET",
        f"{ac.base_url}v2/api/identities",
        match=[
            matchers.query_param_matcher({"ids": closed_idp_identity_document["id"]}),
        ],
        json={"identities": [closed_idp_identity_document]},
    )
    # check/guest_collection
    responses.add(
        "GET",
        f"{ac.base_url}v2/api/identities",
        match=[
            matchers.query_param_matcher({"ids": email_idp_identity_document["id"]}),
        ],
        json={"identities": [email_idp_identity_document]},
    )
    # commands/myproxy
    responses.add(
        "GET",
        f"{ac.base_url}v2/api/identity_providers",
        match=[
            matchers.query_param_matcher(
                {"domains": globus_identity_provider_document["domains"][0]},
            ),
        ],
        json={"identity_providers": [globus_identity_provider_document]},
    )
    # commands/custom
    responses.add(
        "GET",
        f"{ac.base_url}v2/api/identity_providers",
        match=[
            matchers.query_param_matcher(
                {
                    "domains": ",".join(
                        [
                            globus_identity_provider_document["domains"][0],
                            example_identity_provider_document["domains"][0],
                        ]
                    )
                },
            ),
        ],
        json={
            "identity_providers": [
                example_identity_provider_document,
                globus_identity_provider_document,
            ]
        },
    )
    # commands/custom
    responses.add(
        "GET",
        f"{ac.base_url}v2/api/identity_providers",
        match=[
            matchers.query_param_matcher(
                {
                    "domains": ",".join(
                        [
                            example_identity_provider_document["domains"][0],
                            globus_identity_provider_document["domains"][0],
                        ]
                    ),
                },
            ),
        ],
        json={
            "identity_providers": [
                example_identity_provider_document,
                globus_identity_provider_document,
            ]
        },
    )
    # commands/custom
    responses.add(
        "GET",
        f"{ac.base_url}v2/api/identity_providers",
        match=[
            matchers.query_param_matcher(
                {"domains": email_identity_provider_document["domains"][0]},
            ),
        ],
        json={"identity_providers": [email_identity_provider_document]},
    )
    # commands/custom
    responses.add(
        "GET",
        f"{ac.base_url}v2/api/identity_providers",
        match=[matchers.query_param_matcher({"domains": "unknown.org"})],
        json={"identity_providers": []},
    )

    return None
